*==============================================================================
* Program:			Lightbox
* Purpose:			Adds a "lightbox" image to the specified form, visually
*						indicating that the form is unavailable
* Author:			Doug Hennig, adapted from code written by Bernard Bout
* Last Revision:	04/16/2009
* Parameters:		toForm   - the form to add the lightbox to
*					tlEnable - .T. to reenable the form
*					tnColor  - the color to use (optional: if it isn't passed,
*						the system color for an active caption is used; also,
*						not needed if tlEnable is .T.)
* Returns:			.T.
* Environment in:	System.APP is available to be run
* Environment out:	the form has an image added to it if tlEnable is .F. and
*						the image is not visible if tlEnable is .T.
*					System.APP has been run, adding a System object to _screen
*==============================================================================

lparameters toForm, ;
	tlEnable, ;
	tnColor
local llImage, ;
	lnWidth, ;
	lnHeight, ;
	lnColor, ;
	lnFactor, ;
	lnRed, ;
	lnGreen, ;
	lnBlue, ;
	loClrMatrix, ;
	loBmp, ;
	loGfx, ;
	lcFile

* Define some constants.

#define COLOR_ACTIVECAPTION         2

* See if the specified form has a LightBox image.

llImage = pemstatus(toForm, 'imgLightBox', 5)
do case

* It does and we're supposed to enable the form, so hide the image.

	case tlEnable and llImage
		toForm.imgLightBox.Visible    = .F.
		toForm.imgLightBox.PictureVal = ''

* We're supposed to disable the form, so ensure GDIPlusX libraries are open and
* create the image if necessary.

	case not tlEnable
		do System.app
		if not llImage
			toForm.AddObject('imgLightBox', 'Image')
		endif not llImage

* Size the image as necessary.

		lnWidth  = toForm.Width
		lnHeight = toForm.Height
		toForm.imgLightBox.Width  = lnWidth
		toForm.imgLightBox.Height = lnHeight	

* If we weren't passed a color to use, find out the system color for an active
* caption.

		if vartype(tnColor) = 'N'
			lnColor = tnColor
		else
			declare integer GetSysColor in user32 integer nIndex
			lnColor = GetSysColor(COLOR_ACTIVECAPTION)
		endif vartype(tnColor) = 'N'

* Get the colors we'll need.

		lnFactor = 0.90  && 0 = Dark   1 = Bright
		lnRed    = bitand(lnColor, 0x000000FF) / 255 * lnFactor
		lnGreen  = bitrshift(bitand(lnColor, 0x0000FF00), 8) / 255 * lnFactor
		lnBlue   = bitrshift(bitand(lnColor, 0x00FF0000), 16) / 255 * lnFactor

* Create a BMP of the desired color and use it as the picture for the image.
	
		with _Screen.System.Drawing as xfcDrawing of Source\System.PRG
			loClrMatrix = .Imaging.ColorMatrix.New( ;
				lnRed,       0,      0, 0, 0, ;
				    0, lnGreen,      0, 0, 0, ;		
				    0,       0, lnBlue, 0, 0, ;
				    0,       0,      0, 1, 0, ;
				    0,       0,      0, 0, 1)
			loBmp = .Bitmap.FromScreen(toForm.imgLightBox)
			loBmp.ApplyColorMatrix(loClrMatrix)
			loGfx = .Graphics.FromImage(loBmp)
			loGfx.FillRectangle(.SolidBrush.New(.Color.FromARGB(128, 0, 0, 0)), ;
				0, 0, lnWidth, lnHeight)
		endwith
		with toForm.imgLightBox
			.PictureVal = loBmp.GetPictureValFromHBitmap()
			.ZOrder(0)
			.Visible = .T.
		endwith
endcase













